#include "StdAfx.h"
#include ".\mavlpthw.h"

//     inpout32.dll
#pragma comment (lib, "inpout32.lib")
short _stdcall Inp32(short PortAddress);
void _stdcall Out32(short PortAddress, short data);

#pragma comment (exestr, "MAVius 2005, LPT Port control, class mavlpthw")

//       
const short mfunct[12] = { 0x555, 0x333, 0x8E3, 0x3CF, 0x249, 0x111,
							0x41, 0xE0F, 0x8A2, 0x7FC, 0x7FD, 0x75D };


mavLptHW::mavLptHW(void)
{
	FreeZ();
}

mavLptHW::~mavLptHW(void)
{
	FreeZ();
}

bool mavLptHW::Init()
{
	if (this->LocatePort() && this->base_port)
		return true;
	return false;
}

bool mavLptHW::Init(DWORD NumLeds)
{
	if (NumLeds > 12)
		return false;
	if (NumLeds < 1)
		return false;
	this->num_leds = NumLeds;
	return Init();
}

void mavLptHW::FreeZ()
{
	this->base_port = 0;
	this->current_state = 0;
	this->num_leds = 0;
}

// ----------------------------------------------------------------------------
//    LPT      value
void mavLptHW::SetValue(DWORD value)
{
	if (value == this->current_state) return;
	DWORD val1 = LOBYTE(value);
	if (this->num_leds > 8)
	{
		DWORD val2 = (~HIBYTE(value)) & 0xF;
		if (val2 & (1 << 2))
			val2 &= 0xB;
		else
			val2 |= 0x4;
		Out32(LOWORD(base_port), LOWORD(val1));
		Out32(
			LOWORD(base_port) + 2,
			LOWORD(val2) & ((1 << (this->num_leds - 8)) - 1)
			);
	}
	else
	{
		Out32(LOWORD(base_port), LOWORD(val1) & ((1 << this->num_leds) - 1));
	}
	this->current_state = value;
	//  val2    (port + 2)  
	//  .. 3  4-    . :)
}

// ----------------------------------------------------------------------------
//    LPT 
DWORD mavLptHW::GetValue()
{
	DWORD tmp = 0;
	tmp += Inp32(LOWORD(base_port));
	return tmp;
}

// ----------------------------------------------------------------------------
//    (int c)   (short *mdig).
// Compatibility (MAVius 2004, LPT Port console)
bool mavLptHW::DoCommand(int c, DWORD *mdig)
{
	DWORD dig = *mdig;

	// 1,2,3,4,5,6,7,8,9,0,-,=
	if ((c >= '1') && (c <= '9'))
		dig = 1 << (c - '1');

	// F1, F2, ..., F12
	else if ((c >= VK_F1) && (c <= VK_F12))
		dig = mfunct[c - VK_F1];

	else switch (c)
	{
	case VK_UP:
		dig++;
		break;
	case VK_DOWN:
		dig--;
		break;
	case VK_LEFT:
		dig &= (1 << this->num_leds) - 1;
		if (dig & 1) dig |= (1 << this->num_leds);
		dig >>= 1;
		break;
	case VK_RIGHT:
		dig <<= 1;
		if (dig & (1 << this->num_leds)) dig |= 1;
		break;
	case '0':
		dig = 1 << 9;
		break;
	case 0xBD: //  "-"
		dig = 1 << 10;
		break;
	case 0xBB: //  "="
		dig = 1 << 11;
		break;
	case VK_RETURN:
		dig = ~dig;
		break;
	case VK_SPACE:
		if (dig & ((1 << this->num_leds) - 1))
			dig = 0;
		else
			dig = (1 << this->num_leds) - 1;
		break;
	case VK_HOME:
		dig = 0;
		break;
	case VK_END:
		dig = 0xFFF;
	}

	// "    (short *mdig)"
	if (*mdig == dig)
		return false;

	*mdig = dig & ((1 << this->num_leds) - 1);
	return true;
}

DWORD mavLptHW::GetPort()
{
	return this->base_port;
}


// ============================================================================
// Common Functions

// ----------------------------------------------------------------------------
//    .
DWORD mavLptHW::LocatePort()
{
	HKEY hk;
	DWORD info;
	DWORD addr = 0;
	DWORD size = 4;

	//     
	if ( RegCreateKeyEx(HKEY_LOCAL_MACHINE, "Software\\MAVius\\LPT_port",
			NULL, "mLpt_port", REG_OPTION_NON_VOLATILE, KEY_READ, NULL,
			&hk, &info) == ERROR_SUCCESS)
	{
		if (RegQueryValueEx(hk, "addr", NULL, NULL, (LPBYTE)&addr, &size)
			== ERROR_SUCCESS)
		{
			RegCloseKey(hk);
			DWORD port = LOWORD(addr);
			if (port != 0)
				if (TestLPTPort(port))
				{
					this->base_port = port;
					return true;
				}
		}
		else
			RegCloseKey(hk);
	}

	//       
	DWORD port = 0x378;
	if (TestLPTPort(port))
	{
		this->base_port = port;
		return true;
	}

	// "  "
	return false;
}

// ----------------------------------------------------------------------------
//    .       
//   Inp32(addr)      
//   Out32(addr, $random),   0xFFFF.    
//           
//    -    . :)
bool mavLptHW::TestLPTPort(DWORD addr)
{
	DWORD back = Inp32(LOWORD(addr));
	Out32(LOWORD(addr), 0x55);
	DWORD val = Inp32(LOWORD(addr));
	Out32(LOWORD(addr), LOWORD(back));
	this->current_state = LOWORD(val);

	if (val == 0x55)
		return true;
	return false;
}

